public enum DashStyle {
	SOLID,
	DASH,
	DOT,
	DASH_DOT,
	DASH_DOT_DOT
}

static const Gdk.Color blue = {0, 0, 0, 65535};

public abstract class AbstractPlot: Object {
	/* properties */
	public double		line_width	{ get; set; default = 1.0; }
	public Gdk.Color	line_color	{ get; set; default = blue;}
	public DashStyle	line_style	{ get; set; default = DashStyle.SOLID;}
	public bool		visible		{ get; set; default = true;}
	public double		min_arg		{ get; construct set;}
	public double		max_arg		{ get; construct set;}
	
	/* interface methods */
	public abstract void render (Cairo.Context ctx,
	                             double min_x, double max_x,
	                             double min_y, double max_y,
	                             double step_x, double step_y
	                             );
	public abstract bool supports_unlimited_arg ();
}

public abstract class SingleExpressionPlot: AbstractPlot {
	protected Function?	function;
	protected Library	library;
	private string		_expression;
	
	public string expression {
		get { return _expression; }
		set
		{
			library.unregister (function);
			library.register (function = Function.from_string (_expression = value));
		}
	}
	
	private SingleExpressionPlot () {}
	~SingleExpressionPlot ()
	{
		library.unregister (function);
	}
}

public abstract class CoupleExpressionPlot: AbstractPlot {
	protected Function?	function_x;
	protected Function?	function_y;
	protected Library	library;
	private string		_expression_x;
	private string		_expression_y;

	public string expression_x {
		get { return _expression_x; }
		set
		{
			library.unregister (function_x);
			library.register (function_x = Function.from_string (_expression_x = value));
		}
	}

	public string expression_y {
		get { return _expression_y; }
		set
		{
			library.unregister (function_y);
			library.register (function_y = Function.from_string (_expression_y = value));
		}
	}

	private CoupleExpressionPlot () {}
	~CoupleExpressionPlot ()
	{
		library.unregister (function_x);
		library.unregister (function_y);
	}
}
