public class FunctionSin: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "sin"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.sin (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = new FunctionCos ();
	}
}

public class FunctionCos: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "cos"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.cos (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = Function.from_string ("d_cos(x)=-sin");
	}
}

public class FunctionTan: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "tan"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.tan (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = Function.from_string ("d_tan(x)=1/cos^2");
	}
}

private class AbsDerivative: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "d_abs"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return (x < 0) ? -1 : 1; }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = Function.from_string ("d_d_abs(x)=0");
	}
}

public class FunctionAbs: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "abs"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.fabs (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = new AbsDerivative ();
	}
}

public class FunctionLog: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "log"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.log (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = Function.from_string ("d_log(x)=1/x");
	}
}

public class FunctionExp: FunctionIface, Object {
	public unowned string get_name () { return "exp"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.exp (x); }
	public FunctionIface derivate () { return this; }
}

public class FunctionSinh: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "sinh"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.sinh (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = new FunctionCosh ();
	}
}

public class FunctionCosh: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "cosh"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.cosh (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = new FunctionSinh ();
	}
}

public class FunctionTanh: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "tanh"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.tanh (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = Function.from_string ("d_tanh(x)=1/cosh^2");
	}
}

public class FunctionAsin: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "asin"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.asin (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = Function.from_string ("d_asin(x)=1/(1-x^2)^0.5");
	}
}

public class FunctionAcos: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "acos"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.acos (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = Function.from_string ("d_acos(x)=-1/(1-x^2)^0.5");
	}
}

public class FunctionAtan: FunctionIface, Object {
	private FunctionIface? derivative;
	public unowned string get_name () { return "atan"; }
	public bool link_with (Library library) { return true; }
	public double evaluate (double x) { return Math.atan (x); }
	public FunctionIface derivate () {
		if (likely (derivative != null))
			return derivative;
		else
			return derivative = Function.from_string ("d_atan(x)=1/(1+x^2)");
	}
}

