public class CartesianPlot: SingleExpressionPlot {
	
	public CartesianPlot (Library library)
	{
		Object (min_arg: -double.MAX, max_arg: double.MAX);
		this.library = library;
	}

	public override bool supports_unlimited_arg ()
	{
		return true;
	}
	
	public override void render (Cairo.Context ctx,
	                             double min_x, double max_x,
	                             double min_y, double max_y,
	                             double step_x, double step_y
	                             )
	{
		min_x = Math.fmax (min_x, min_arg);
		max_x = Math.fmin (max_x, max_arg);
		if (function == null || !function.link_with (library) || max_x < min_x)
			return;
		
		bool move_to = true;
		for (double x = min_x; x <= max_x; x += step_x) {
			double y = function.evaluate (x);
			if (Math.isnan (y) != 0 || Math.isinf (y) != 0) {
				move_to = true;
				continue;
			}
			
			if (move_to) {
				ctx.move_to (x, y);
				move_to = false;
			} else {
				ctx.line_to (x, y);
			}
		}
	}
}
