using Gtk;

public void cairo_set_dash_style (Cairo.Context ctx, DashStyle style, double offset)
{
	double d1 = 1.0, d2 = 5.0, tmp = 0.0;
	ctx.device_to_user_distance (ref d1, ref tmp);
	ctx.device_to_user_distance (ref d2, ref tmp);
	switch (style) {
	case DashStyle.SOLID:
		ctx.set_dash ({d1 * 1e7, 0.0}, offset);
		break;
	case DashStyle.DASH:
		ctx.set_dash ({d2, d2}, offset);
		break;
	case DashStyle.DOT:
		ctx.set_dash ({d1, d2}, offset);
		break;
	case DashStyle.DASH_DOT:
		ctx.set_dash ({d2, d2, d1, d2}, offset);
		break;
	case DashStyle.DASH_DOT_DOT:
		ctx.set_dash ({d2, d2, d1, d2, d1, d2}, offset);
		break;
	}
}

public class CellRendererDash: CellRenderer {
	public DashStyle dash {get; set;}
	
	public CellRendererDash ()
	{
		GLib.Object ();
	}
	
	public override void get_size (Widget widget,
	                               Gdk.Rectangle? cell_area,
	                               out int x_offset,
	                               out int y_offset,
	                               out int width,
	                               out int height)
	{
		if (&x_offset != null) x_offset = 5;
		if (&y_offset != null) y_offset = 5;
		if (&width    != null) width    = widget.allocation.width;
		if (&height   != null) height   = 15;
	}
	
	public override void render (Gdk.Window window,
	                             Gtk.Widget widget,
	                             Gdk.Rectangle background_area,
	                             Gdk.Rectangle cell_area,
	                             Gdk.Rectangle expose_area,
	                             CellRendererState flags)
	{
		var ctx = Gdk.cairo_create (window);
		Gdk.cairo_rectangle (ctx, expose_area);
		ctx.clip ();
		
		double x1 = cell_area.x + 3;
		double y1 = cell_area.y + cell_area.height / 2;
		double x2 = x1 + cell_area.width - 6;
		double y2 = y1;
		
		ctx.device_to_user (ref x1, ref y1);
		ctx.device_to_user (ref x2, ref y2);
		ctx.set_line_width (2.0);
		ctx.set_line_cap (Cairo.LineCap.ROUND);
		cairo_set_dash_style (ctx, dash, 0.0);
		ctx.move_to (x1, y1);
		ctx.line_to (x2, y2);
		ctx.stroke ();
	}
}

