public class Library: Object {
	private HashTable <string, void*> library;
	
	static void value_free (string key, owned SList list)
	{
		weak SList <FunctionIface> ? cur = list;
		while (cur != null) {
			((FunctionIface) cur.data).unref ();
			cur = cur.next;
		}
		
		list = null;
	}
	
	public Library ()
	{
		library = new HashTable <string, void*>.full (str_hash, str_equal, free, null);
	}
	
	~Library ()
	{
		library.for_each ((HFunc) value_free);
	}
	
	public void register (FunctionIface? function)
	{
		if (function != null) {
			weak SList <FunctionIface> list = (SList) library.lookup (function.get_name ());
			list.prepend (function);
			library.insert (function.get_name (), list);
		}
	}
	
	public void unregister (FunctionIface? function)
	{
		if (function != null) {
			weak SList <FunctionIface> list = (SList) library.lookup (function.get_name ());
			list.remove_all (function);
			library.insert (function.get_name (), list);
		}
	}
	
	public unowned FunctionIface? lookup (string name)
	{
		weak SList <FunctionIface> list = (SList) library.lookup (name);
		if (list != null)
			return (FunctionIface) list.data;
		else
			return null;
	}
	
	private const string[] good_name = {"f", "g", "h", "φ", "ψ", "ω", "σ", "ξ"};
	public string suggest_free_name (bool parametric = false)
	{
		for (int i = 0; i < good_name.length; i++) {
			string attempt = good_name[i];
			if (parametric) {
				if (library.lookup (attempt + "_x") == null &&
				    library.lookup (attempt + "_y") == null)
					return attempt;
			} else {
				if (library.lookup (attempt) == null)
					return attempt;
			}
		}
		
		int mod = 'z' - 'f' + 1;
		for (int i = 1; true; i++) {
			int len, tmp;
			
			len = 0; tmp = i;
			while (tmp > 0) {
				len++;
				tmp /= mod;
			}
			
			char* name = new char [len+1];
			name[len] = 0;
			len = 0; tmp = i;
			while (tmp > 0) {
				name[len++] = (char) (tmp % mod + 'f');
				tmp /= mod;
			}
			
			string sname = (string) (owned) name;
			if (parametric) {
				if (library.lookup (sname + "_x") == null &&
				    library.lookup (sname + "_y") == null)
					return sname;
			} else {
				if (library.lookup (sname) == null)
					return sname;
			}
		}
	}
}

