weak string ui_dir;
weak string locale_dir;
weak string prg_path;
const string app_name = "vPlot";

[CCode (array_length = false, array_null_terminated = true)]
static string[] arguments;
const OptionEntry[] options = {
	{"", 0, 0, OptionArg.FILENAME_ARRAY, ref arguments, null, "URI"},
	{ null }
};

int main (string[] args)
{
	prg_path = args[0];
	ui_dir = Environment.get_variable ("UI_DIR");
	if (ui_dir == null) ui_dir = Config.PKGDATADIR;
	locale_dir = Environment.get_variable ("LOCALE_DIR");
	if (locale_dir == null) locale_dir = Config.LOCALEDIR;
	
	Intl.bindtextdomain (Config.GETTEXT_PACKAGE, locale_dir);
	Intl.bind_textdomain_codeset (Config.GETTEXT_PACKAGE, "UTF-8");
	Intl.textdomain (Config.GETTEXT_PACKAGE);
	
	Environment.set_application_name (app_name);
	try {
		var opt_context = new OptionContext ("");
		opt_context.set_help_enabled (true);
		opt_context.add_main_entries (options, null);
		opt_context.add_group (Gtk.get_option_group (true));
		opt_context.parse (ref args);
	} catch (OptionError e) {
		stdout.printf ("%s\n", e.message);
		stdout.printf (_("Run '%s --help' to see a full list of available command line options.\n"), args[0]);
		return 1;
	}
	
	if (arguments.length > 1) {
		stdout.printf (_("Too much arguments\n"));
		return 1;
	}
	
	register_transform_functions ();
	Gtk.init (ref args);
	Gtk.Window.set_default_icon_name (Config.PACKAGE_NAME);
	
	PlotWindow plot_window;
	int ret;
	try {
		if (arguments != null && arguments[0] != null)
			plot_window = new PlotWindow (File.new_for_commandline_arg (arguments[0])).run ();
		else
			plot_window = new PlotWindow ().run ();
		Gtk.main ();
	} catch (Error e) {
		show_error (_("Error loading %s").printf (app_name), e.message, null);
		ret = 1;
	} finally {
		ret = 0;
	}
	
	plot_window = null; /* HACK */
	g_strfreev (arguments);
	return ret;
}

