public class ParametricPlot: CoupleExpressionPlot {
	
	public ParametricPlot (Library library)
	{
		Object (min_arg: -10.0, max_arg: 10.0);
		this.library = library;
	}
	
	public override bool supports_unlimited_arg ()
	{
		return false;
	}

	public override void render (Cairo.Context ctx,
	                             double min_x, double max_x,
	                             double min_y, double max_y,
	                             double step_x, double step_y
	                             )
	{
		if (function_x == null || function_y == null)
			return;
		if (!function_x.link_with (library) || !function_y.link_with (library))
			return;
		if (max_arg <= min_arg)
			return;

		bool move_to = true;
		double dt = (max_arg - min_arg) / 1e4; /* FIXME */
		for (double t = min_arg; t <= max_arg; t += dt) {
			double x = function_x.evaluate (t);
			double y = function_y.evaluate (t);
			if (Math.isnan (x) != 0 || Math.isinf (x) != 0 ||
				Math.isnan (y) != 0 || Math.isinf (y) != 0)
			{
				move_to = true;
				continue;
			}
			
			if (move_to) {
				ctx.move_to (x, y);
				move_to = false;
			} else {
				ctx.line_to (x, y);
			}
		}
	}
}

