public class PolarPlot: SingleExpressionPlot {

	public PolarPlot (Library library)
	{
		Object (min_arg: 0.0, max_arg: 2*Math.PI);
		this.library = library;
	}

	public override bool supports_unlimited_arg ()
	{
		return false;
	}
	
	public override void render (Cairo.Context ctx,
	                             double min_x, double max_x,
	                             double min_y, double max_y,
	                             double step_x, double step_y
	                             )
	{
		if (function == null || !function.link_with (library) || max_arg <= min_arg)
			return;
		
		bool move_to = true;
		double eps = 1e-7;
		double dt = (max_arg - min_arg) / 1e4; /* FIXME */
		for (double phi = min_arg; phi <= max_arg; phi += dt) {
			double r = function.evaluate (phi);
			if (Math.isnan (r) != 0 || Math.isinf (r) != 0 || r < -eps) {
				move_to = true;
				continue;
			}
			if (r < 0.0) r = 0.0;
			
			double x = r * Math.cos (phi);
			double y = r * Math.sin (phi);
			if (move_to) {
				ctx.move_to (x, y);
				move_to = false;
			} else {
				ctx.line_to (x, y);
			}
		}
	}
}
