static void color_to_string (Value src, out Value dst)
{
	Gdk.Color* color = (Gdk.Color*) src.get_boxed ();
	dst.set_string (color->to_string ());
}

static void string_to_color (Value src, out Value dst)
{
	Gdk.Color color;
	Gdk.Color.parse (src.get_string (), out color);
	dst.set_boxed (&color);
}

static void enum_to_string (Value src, out Value dst)
{
	EnumClass klass = (EnumClass) src.type ().class_ref ();
	unowned EnumValue? enum_value = klass.get_value (src.get_enum ());
	
	if (enum_value != null)
		dst.set_string (enum_value.value_nick);
	else
		dst.take_string ("%ld".printf (src.get_enum ()));
}

static void string_to_enum (Value src, out Value dst)
{
	EnumClass klass = (EnumClass) dst.type ().class_ref ();
	unowned EnumValue? enum_value = klass.get_value_by_nick (src.get_string ());
	
	if (enum_value != null)
		dst.set_enum (enum_value.value);
	else
		dst.set_enum (0);
}

static void string_to_double (Value src, out Value dst)
{
	switch (src.get_string ()) {
	case "MAX":
		dst.set_double (double.MAX);
		break;
	case "MIN":
		dst.set_double (-double.MAX);
		break;
	default:
		double val;
		src.get_string ().scanf ("%lf", out val);
		dst.set_double (val);
		break;
	}
}

static void double_to_string (Value src, out Value dst)
{
	double val = src.get_double ();
	if (val == double.MAX)
		dst.set_string ("MAX");
	else if (val == -double.MAX)
		dst.set_string ("MIN");
	else 
		dst.take_string ("%lf".printf (src.get_double ()));
}

static void string_to_boolean (Value src, out Value dst)
{
	dst.set_boolean (src.get_string () == "TRUE");
}

void register_transform_functions ()
{
	Value.register_transform_func (typeof (Gdk.Color),
				       typeof (string),
				       (ValueTransform) color_to_string);
	Value.register_transform_func (Type.ENUM,
				       typeof (string),
				       (ValueTransform) enum_to_string);
	Value.register_transform_func (typeof (string),
				       Type.ENUM,
				       (ValueTransform) string_to_enum);
	Value.register_transform_func (typeof (double),
				       typeof (string),
				       (ValueTransform) double_to_string);
	Value.register_transform_func (typeof (string),
				       typeof (double),
				       (ValueTransform) string_to_double);
	Value.register_transform_func (typeof (string),
				       typeof (bool),
				       (ValueTransform) string_to_boolean);
	Value.register_transform_func (typeof (string),
				       typeof (Gdk.Color),
				       (ValueTransform) string_to_color);
}

int gcd (int a, int b)
{
	while (b != 0) {
		int t = a;
		a = b;
		b = t % b;
	}
	return a;
}

public void show_error (string title, string description, Gtk.Window? parent = null)
{
	var dialog = new Gtk.MessageDialog (parent,
					    Gtk.DialogFlags.MODAL,
					    Gtk.MessageType.ERROR,
					    Gtk.ButtonsType.OK,
					    "%s", title);
	dialog.format_secondary_text (description);
	dialog.run ();
	dialog.destroy ();
}

extern void g_strfreev (char **str_array);

