static Library library;
static int test_num = 0;

static bool simple_test (string expr, double arg, double expected, double eps)
{
	stdout.printf("Test %d: ", ++test_num);
	Function? func = Function.from_string ("y(x)=" + expr);
	if (func == null) {
		stdout.printf ("FAIL: wrong syntax\n");
		stdout.flush ();
		return false;
	}
	if (!func.link_with (library)) {
		stdout.printf ("FAIL: link failed\n");
		stdout.flush ();
		return false;
	}
	double ret = func.evaluate (arg);
	if (Math.fabs (ret - expected) > eps) {
		stdout.printf ("FAIL: %lf expected, but %lf returned\n", expected, ret);
		stdout.flush ();
		return false;
	} else {
		stdout.printf ("OK\n");
		stdout.flush ();
		return true;
	}
}

int main (string[] arg)
{
	library = new Library ();
	
	simple_test ("1e2", -2, 100.0, 1e-7);
	simple_test ("x^2", -2, 4.0, 1e-7);
	simple_test ("0.1e1-5*x^(2-x+x)^1", 3.0, -44.0, 1e-7);
	simple_test ("2^3^2", 3.0, 512.0, 1e-7);
	
	/* library.register (new SimpleFunction ("sin", (x) => {return Math.sin (x);}));
	simple_test ("sin(x)", 1.5, 0.99749498660405, 1e-7);
	library.register (new SimpleFunction ("log", (x) => {return Math.log (x);}));
	simple_test ("x*sin(log)^2", 3.0, 2.379382401437029, 1e-7);*/
	
	Function g = Function.from_string ("g(x) = 1");
	library.register (Function.from_string ("g(x) = x"));
	library.unregister (g);
	library.register (Function.from_string ("f(x) = g^2"));
	simple_test ("g*f(g(g(g)))", 2.0, 8.0, 1e-7);
	return 0;
}
